local M = LibStub("LibSharedMedia-3.0")

--Tables--
local mode = {"Experience", "Reputation"}
local FullAlign = {
	TOPLEFT = "TOPLEFT",
	TOP = "TOP",
	TOPRIGHT = "TOPRIGHT",
	LEFT = "LEFT",
	CENTER = "CENTER",
	RIGHT = "RIGHT",
	BOTTOMLEFT = "BOTTOMLEFT",
	BOTTOM = "BOTTOM",
	BOTTOMRIGHT = "BOTTOMRIGHT"
}
local AlignH = {LEFT = "LEFT", CENTER = "CENTER", RIGHT = "RIGHT"}
local AlignV = {BOTTOM = "BOTTOM", CENTER = "CENTER", TOP = "TOP"}
local fontflags = {
	NONE = "None", 
	OUTLINE = "Outline", 
	THICKOUTLINE = "Thick Outline", 
	["NONE, MONOCHROME"] = "No Outline, Monochrome", 
	["OUTLINE, MONOCHROME"] = "Outline, Monochrome", 
	["THICKOUTLINE, MONOCHROME"] = "Thick Outline, Monochrome"
}

local Defaults = {
	profile = {
		scale = 1,
		mode = 2,
		x = 0,
		y = -300,
		--[[Top Text]]--
		toptext = true,
		topfont = "Accidental Presidency",
		toptextsize = 12,
		toptextflags = "OUTLINE",
		toptextanchor = "BOTTOM",
		toptextx = 0,
		toptexty = 0,
		toptextHalign = "CENTER",
		toptextcolor = {
			r = 1,
			g = 1,
			b = 1
		},
		--[[Bottom Text]]--
		bottomtext = true,
		bottomfont = "Accidental Presidency",
		bottomtextsize = 12,
		bottomtextflags = "OUTLINE",		
		bottomtextanchor = "BOTTOM",
		bottomtextx = 0,
		bottomtexty = 0,
		bottomtextHalign = "CENTER",
		bottomtextcolor = {
			r = 1,
			g = 1,
			b = 1
		},
		texture = "FastLevelsDefault",
		bartexture = "FastLevelsDefault",
		width = 961,
		height = 17,
		dotheight = 6,
		barcolor = {
			r = 0,
			g = 1,
			b = 0
		},
		dotcolor = {
			r = 0,
			g = 1,
			b = 0
		},
		barBGcolor = {
			r = 0.25,
			g = 0.25,
			b = 0.25
		},
		dotBGcolor = {
			r = 0.25,
			g = 0.25,
			b = 0.25
		},
	}
}

local function GetValue(info)
	local val = info.arg[1]
	return FastLevels.db.profile[val]
end

local function SetValue(info,v)
	local val = info.arg[1]
	FastLevels.db.profile[val] = v
	FastLevelsUpdate()
end

local function GetColor(info)
	local val = info.arg[1]
	local c = FastLevels.db.profile[val]
	return c.r,c.g,c.b
end

local function SetColor(info,r,g,b)
	local val = info.arg[1]
	local c = FastLevels.db.profile[val]
	c.r,c.g,c.b = r,g,b
	FastLevelsUpdate()
end

local options = {
	type = "group",
	order = 1,
	name = "Options",
	args = {
		Mode = {
			type = "select",
			name = "Mode",
			order = 0,
			values = mode,
			get = GetValue,
			set = SetValue,
			arg = {"mode"}
		},
		header1 = {
			type = "header",
			order = 0.5,
			name = GetAddOnMetadata("FastLevels", "Title").." v"..GetAddOnMetadata("FastLevels", "Version"),
		},
		Placement = {
			type = "group",
			order = 1,
			name = "Placement",
			args = {
				position_x = {
					type = "range",
					width = "full",
					order = 1,
					name = "X",
					get = GetValue,
					set = SetValue,
					step = 1,
					min = -1200,
					max = 1200,
					arg = {"x"},
				},
				position_y = {
					type = "range",
					width = "full",
					order = 2,
					name = "Y",
					get = GetValue,
					set = SetValue,
					step = 1,
					min = -1200,
					max = 1200,
					arg = {"y"},
				},
				scale = {
					type = "range",
					width = "full",
					order = 3,
					name = "Scale",
					get = GetValue,
					set = SetValue,
					step = 0.01,
					min = 0.75,
					max = 1.25,
					isPercent = true,
					arg = {"scale"},
				},
			},
		},
		BarSettings = {
			type = "group",
			order = 2,
			name = "Bar Settings",
			args = {
				texture = {
					type = "select",
					order = 5,
					width = "full",
					name = "Dot Texture",
					dialogControl = "LSM30_Statusbar",
					values = AceGUIWidgetLSMlists.statusbar,
					get = GetValue,
					set = SetValue,
					arg = {"texture"},
				},
				dotheight = {
					type = "range",
					width = "full",
					order = 6,
					name = "Dot Height",
					get = GetValue,
					set = SetValue,
					step = 1,
					min = 2,
					max = 25,
					arg = {"dotheight"},
				},
				dotcolor = {
					type = "color",
					width = "full",
					order = 7,
					name = "dotcolor",
					get = GetColor,
					set = SetColor,
					arg = {"dotcolor"},
				},
				dotBGcolor = {
					type = "color",
					width = "full",
					order = 8,
					name = "dotBGcolor",
					get = GetColor,
					set = SetColor,
					arg = {"dotBGcolor"},
				},
				header2 = {
					type = "header",
					order = 9,
					name = "Bar Settings",
				},
				bartexture = {
					type = "select",
					order = 10,
					width = "full",
					name = "Bar Texture",
					dialogControl = "LSM30_Statusbar",
					values = AceGUIWidgetLSMlists.statusbar,
					get = GetValue,
					set = SetValue,
					arg = {"bartexture"},
				},		
				width = {
					type = "range",
					width = "full",
					order = 11,
					name = "Bar Width",
					get = GetValue,
					set = SetValue,
					step = 20,
					min = 21,
					max = 2301,
					arg = {"width"},
				},
				height = {
					type = "range",
					width = "full",
					order = 12,
					name = "Bar Height",
					get = GetValue,
					set = SetValue,
					step = 1,
					min = 3,
					max = 25,
					arg = {"height"},
				},
				barcolor = {
					type = "color",
					width = "full",
					order = 13,
					name = "Bar Color",
					get = GetColor,
					set = SetColor,
					arg = {"barcolor"},
				},
				barBGcolor = {
					type = "color",
					width = "full",
					order = 14,
					name = "Bar Background Color",
					get = GetColor,
					set = SetColor,
					arg = {"barBGcolor"},
				},
			},
		},
		TextSettings = {
			type = "group",
			name = "Text Settings",
			order = 3,
			args = {
				TopTextSettings = {
					type = "group",
					order = 1,
					name = "Top Text Settings",
					args = {
						enabletext = {
							type = "toggle",
							name = "Show Text",
							width = "full",
							order = 1,
							get = GetValue,
							set = SetValue,
							arg = {"toptext"}		
						},
						font = {
							type = "select",
							name = "Font",
							width = "full",
							order = 2,
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							get = GetValue,
							set = SetValue,
							arg = {"topfont"},
						},
						flags = {
							type = "select",
							name = "Flags",
							width = "full",
							order = 3,
							values = fontflags,
							get = GetValue,
							set = SetValue,
							arg = {"toptextflags"},
						},
						color = {
							type = "color",
							name = "Color",
							width = "full",
							order = 4,
							get = GetColor,
							set = SetColor,
							arg = {"toptextcolor"},
						},
						size = {
							type = "range",
							name = "size",
							order = 5,
							min = 4,
							max = 32,
							get = GetValue,
							set = SetValue,
							arg = {"toptextsize"},
						},
						x = {
							type = "range",
							name = "x",
							order = 6,
							min = -120,
							max = 120,
							get = GetValue,
							set = SetValue,
							arg = {"toptextx"}
						},
						y = {
							type = "range",
							name = "y",
							order = 7,
							min = -120,
							max = 120,
							get = GetValue,
							set = SetValue,
							arg = {"toptexty"}
						},
						anchor = {
							type = "select",
							name = "Anchoring",
							order = 8,
							values = FullAlign,
							get = GetValue,
							set = SetValue,
							arg = {"toptextanchor"},
						},
					},
				},
				BottomTextSettings = {
					type = "group",
					order = 2,
					name = "Bottom Text Settings",
					args = {
						enabletext = {
							type = "toggle",
							name = "Show Text",
							width = "full",
							order = 1,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtext"}		
						},
						font = {
							type = "select",
							name = "Font",
							width = "full",
							order = 2,
							dialogControl = 'LSM30_Font',
							values = AceGUIWidgetLSMlists.font,
							get = GetValue,
							set = SetValue,
							arg = {"bottomfont"},
						},
						flags = {
							type = "select",
							name = "Flags",
							width = "full",
							order = 3,
							values = fontflags,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtextflags"},
						},					
						color = {
							type = "color",
							name = "Color",
							width = "full",
							order = 4,
							get = GetColor,
							set = SetColor,
							arg = {"toptextcolor"},
						},
						size = {
							type = "range",
							name = "size",
							order = 5,
							min = 4,
							max = 32,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtextsize"},
						},
						x = {
							type = "range",
							name = "x",
							order = 6,
							min = -120,
							max = 120,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtextx"}
						},
						y = {
							type = "range",
							name = "y",
							order = 7,
							min = -120,
							max = 120,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtexty"}
						},
						anchor = {
							type = "select",
							name = "Anchoring",
							order = 8,
							values = FullAlign,
							get = GetValue,
							set = SetValue,
							arg = {"bottomtextanchor"},
						},
					},
				},
			},
		},
		About = {
			type = "group",
			name = "About",
			order = 4,
			args = {
				about = {
					type = "description",
					name = "Simple exp / rep bar alternative to blizzard created by Suicidal Katt. For suggestions and feedback, email to suicidalkatt@gmail.com\n\n",
					order = 1,
				},
				about2 = {
					type = "description",
					name = "Commands:\n/fl or /fastlevels\n  config -- Opens options.\n  toggle -- Switches modes between reputation and experience.\n  about -- Displays about information.\n  help -- Shows available commands.",
					order = 2,
				},
				about3 = {
					type = "description",
					name = "",
					order = 3,
				},
			},
		},
	},
}

function FastLevels:RegisterOptions()
	local db = LibStub('AceDB-3.0'):New('FastLevelsDB', Defaults, 'Default')
	self.db = db
	
	local RegisterCallback = db.RegisterCallback
	
	RegisterCallback(self, 'OnProfileChanged', FastLevelsUpdate)
	RegisterCallback(self, 'OnProfileCopied', FastLevelsUpdate)
	RegisterCallback(self, 'OnProfileReset', FastLevelsUpdate)
	
	local profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)
	local AceConfig = LibStub('AceConfigRegistry-3.0')
	local dialog = LibStub('AceConfigDialog-3.0')
	
	AceConfig:RegisterOptionsTable("FastLevels", options)
	AceConfig:RegisterOptionsTable("FastLevels Profiles",  profile)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FastLevels", "FastLevels")
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FastLevels Profiles", 'Profiles', "FastLevels")
	
	local commands = {}	
	commands.toggle = function()
		if FastLevels.db.profile.mode == 1 then
			FastLevels.db.profile.mode = 2
		else
			FastLevels.db.profile.mode = 1
		end
		FastLevelsUpdate()
	end	
	commands.about = function()
		print("Simple exp / rep bar alternative to blizzard created by Suicidal Katt. For suggestions and feedback, email to suicidalkatt@gmail.com")
	end	
	commands.help = function()
		print("Available commands are /fl or /fastlevels 'config','toggle','help', or 'about'.")
	end	
	commands.config = function()
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	end
	SLASH_FASTLEVELS1,SLASH_FASTLEVELS2 = "/fl", "/fastlevels"
	SlashCmdList["FASTLEVELS"] = function(arg)
		if type(commands[arg]) == 'function' then 
			commands[arg]()
		else 
			print("Available commands are /fl or /fastlevels 'config','toggle','help', or 'about'.")
		end
	end
	--[[
	self:RegisterChatCommand("fl", function () InterfaceOptionsFrame_OpenToCategory(self.optionsFrame) end)
	self:RegisterChatCommand("fastlevels", function () InterfaceOptionsFrame_OpenToCategory(self.optionsFrame) end)
	]]--
end